@@ -11,6 +11,7 @@
 # Licensed under the Eiffel Forum License 2.
 
 from __future__ import unicode_literals, absolute_import, print_function, division
+
 import locale
 import sys
 loc = locale.getlocale()
@@ -29,7 +30,7 @@ import time
 import traceback
 import signal
 
-__version__ = '6.5.1'
+__version__ = '6.6.0'
 
 
 def _version_info(version=__version__):
@@ -51,6 +52,8 @@ def _version_info(version=__version__):
     version_type = namedtuple('version_info',
                               'major, minor, micro, releaselevel, serial')
     return version_type(major, minor, micro, level, serial)
+
+
 version_info = _version_info()
 
 
@@ -65,7 +68,7 @@ def run(config, pid_file, daemon=False):
                'work properly.')
 
     def signal_handler(sig, frame):
-        if sig == signal.SIGUSR1 or sig == signal.SIGTERM:
+        if sig == signal.SIGUSR1 or sig == signal.SIGTERM or sig == signal.SIGINT:
             stderr('Got quit signal, shutting down.')
             p.quit('Closing')
     while True:
@@ -75,15 +78,17 @@ def run(config, pid_file, daemon=False):
                 signal.signal(signal.SIGUSR1, signal_handler)
             if hasattr(signal, 'SIGTERM'):
                 signal.signal(signal.SIGTERM, signal_handler)
+            if hasattr(signal, 'SIGINT'):
+                signal.signal(signal.SIGINT, signal_handler)
             sopel.logger.setup_logging(p)
             p.run(config.core.host, int(config.core.port))
         except KeyboardInterrupt:
             break
-        except Exception:
+        except Exception:  # TODO: Be specific
             trace = traceback.format_exc()
             try:
                 stderr(trace)
-            except:
+            except Exception:  # TODO: Be specific
                 pass
             logfile = open(os.path.join(config.core.logdir, 'exceptions.log'), 'a')
             logfile.write('Critical exception in core')
