@@ -94,7 +94,7 @@ class Sopel(irc.Bot):
         """A dictionary of channels to their users and privilege levels
 
         The value associated with each channel is a dictionary of
-        :class:`sopel.tools.Identifier`\s to
+        :class:`sopel.tools.Identifier`\\s to
         a bitwise integer value, determined by combining the appropriate
         constants from :mod:`sopel.module`.
 
@@ -127,6 +127,9 @@ class Sopel(irc.Bot):
         modules. See :class:`sopel.tools.Sopel.SopelMemory`
         """
 
+        self.shutdown_methods = []
+        """List of methods to call on shutdown"""
+
         self.scheduler = sopel.tools.jobs.JobScheduler(self)
         self.scheduler.start()
 
@@ -217,15 +220,20 @@ class Sopel(irc.Bot):
             # TODO this should somehow find the right job to remove, rather than
             # clearing the entire queue. Issue #831
             self.scheduler.clear_jobs()
-        if (getattr(obj, '__name__', None) == 'shutdown'
-                and obj in self.shutdown_methods):
+        if (getattr(obj, '__name__', None) == 'shutdown' and
+                    obj in self.shutdown_methods):
             self.shutdown_methods.remove(obj)
 
     def register(self, callables, jobs, shutdowns, urls):
-        self.shutdown_methods = shutdowns
+        # Append module's shutdown function to the bot's list of functions to
+        # call on shutdown
+        self.shutdown_methods += shutdowns
         for callbl in callables:
-            for rule in callbl.rule:
-                self._callables[callbl.priority][rule].append(callbl)
+            if hasattr(callbl, 'rule'):
+                for rule in callbl.rule:
+                    self._callables[callbl.priority][rule].append(callbl)
+            else:
+                self._callables[callbl.priority][re.compile('.*')].append(callbl)
             if hasattr(callbl, 'commands'):
                 module_name = callbl.__module__.rsplit('.', 1)[-1]
                 # TODO doc and make decorator for this. Not sure if this is how
@@ -468,7 +476,7 @@ class Sopel(irc.Bot):
 
         try:
             exit_code = func(sopel, trigger)
-        except Exception:
+        except Exception:  # TODO: Be specific
             exit_code = None
             self.error(trigger)
 
@@ -513,9 +521,15 @@ class Sopel(irc.Bot):
 
                     if event not in func.event:
                         continue
-                    if (hasattr(func, 'intents') and
-                            trigger.tags.get('intent') not in func.intents):
-                        continue
+                    if hasattr(func, 'intents'):
+                        if not trigger.tags.get('intent'):
+                            continue
+                        match = False
+                        for intent in func.intents:
+                            if intent.match(trigger.tags.get('intent')):
+                                match = True
+                        if not match:
+                            continue
                     if func.thread:
                         targs = (func, wrapper, trigger)
                         t = threading.Thread(target=self.call, args=targs)
