@@ -12,7 +12,6 @@ dispatch function in bot.py and making it easier to maintain.
 # Licensed under the Eiffel Forum License 2.
 from __future__ import unicode_literals, absolute_import, print_function, division
 
-
 from random import randint
 import re
 import sys
@@ -50,7 +49,7 @@ def auth_after_register(bot):
             'AUTHSERV auth',
             account + ' ' + password
         ))
-    
+
     elif bot.config.core.auth_method == 'Q':
         account = bot.config.core.auth_username
         password = bot.config.core.auth_password
@@ -59,6 +58,13 @@ def auth_after_register(bot):
             account + ' ' + password
         ))
 
+    elif bot.config.core.auth_method == 'userserv':
+        userserv_name = bot.config.core.auth_target or 'UserServ'
+        account = bot.config.core.auth_username
+        password = bot.config.core.auth_password
+        bot.msg(userserv_name, "LOGIN {account} {password}".format(
+                account=account, password=password))
+
 
 @sopel.module.event(events.RPL_WELCOME, events.RPL_LUSERCLIENT)
 @sopel.module.rule('.*')
@@ -165,8 +171,8 @@ def handle_names(bot, trigger):
     """Handle NAMES response, happens when joining to channels."""
     names = trigger.split()
 
-    #TODO specific to one channel type. See issue 281.
-    channels = re.search('(#\S*)', trigger.raw)
+    # TODO specific to one channel type. See issue 281.
+    channels = re.search(r'(#\S*)', trigger.raw)
     if not channels:
         return
     channel = Identifier(channels.group(1))
@@ -175,6 +181,8 @@ def handle_names(bot, trigger):
 
     # This could probably be made flexible in the future, but I don't think
     # it'd be worth it.
+    # If this ever needs to be updated, remember to change the mode handling in
+    # the WHO-handler functions below, too.
     mapping = {'+': sopel.module.VOICE,
                '%': sopel.module.HALFOP,
                '@': sopel.module.OP,
@@ -229,7 +237,13 @@ def track_modes(bot, trigger):
         else:
             arg = Identifier(arg)
             for mode in modes:
-                priv = bot.privileges[channel].get(arg, 0)
+                priv = bot.channels[channel].privileges.get(arg, 0)
+                # Throw an exception if the two privilege-tracking data
+                # structures get out of sync. That should never happen.
+                # This is a good place to check that bot.channels is doing
+                # what it's supposed to do before ultimately removing the old,
+                # deprecated bot.privileges structure completely.
+                assert priv == bot.privileges[channel].get(arg, 0)
                 value = mapping.get(mode[1])
                 if value is not None:
                     if mode[0] == '+':
@@ -237,6 +251,7 @@ def track_modes(bot, trigger):
                     else:
                         priv = priv & ~value
                     bot.privileges[channel][arg] = priv
+                    bot.channels[channel].privileges[arg] = priv
 
 
 @sopel.module.rule('.*')
@@ -482,7 +497,7 @@ def recieve_cap_ls_reply(bot, trigger):
     auth_caps = ['account-notify', 'extended-join', 'account-tag']
     for cap in auth_caps:
         if cap not in bot._cap_reqs:
-            bot._cap_reqs[cap] = [_CapReq('=', 'coretasks', acct_warn)]
+            bot._cap_reqs[cap] = [_CapReq('', 'coretasks', acct_warn)]
 
     for cap, reqs in iteritems(bot._cap_reqs):
         # At this point, we know mandatory and prohibited don't co-exist, but
@@ -544,7 +559,7 @@ def sasl_success(bot, trigger):
     bot.write(('CAP', 'END'))
 
 
-#Live blocklist editing
+# Live blocklist editing
 
 
 @sopel.module.commands('blocks')
@@ -655,10 +670,11 @@ def recv_whox(bot, trigger):
         return LOGGER.warning('While populating `bot.accounts` a WHO response was malformed.')
     _, _, channel, user, host, nick, status, account = trigger.args
     away = 'G' in status
-    _record_who(bot, channel, user, host, nick, account, away)
+    modes = ''.join([c for c in status if c in '~&@%+'])
+    _record_who(bot, channel, user, host, nick, account, away, modes)
 
 
-def _record_who(bot, channel, user, host, nick, account=None, away=None):
+def _record_who(bot, channel, user, host, nick, account=None, away=None, modes=None):
     nick = Identifier(nick)
     channel = Identifier(channel)
     if nick not in bot.users:
@@ -669,9 +685,18 @@ def _record_who(bot, channel, user, host, nick, account=None, away=None):
     else:
         user.account = account
     user.away = away
+    priv = 0
+    if modes:
+        mapping = {'+': sopel.module.VOICE,
+           '%': sopel.module.HALFOP,
+           '@': sopel.module.OP,
+           '&': sopel.module.ADMIN,
+           '~': sopel.module.OWNER}
+        for c in modes:
+            priv = priv | mapping[c]
     if channel not in bot.channels:
         bot.channels[channel] = Channel(channel)
-    bot.channels[channel].add_user(user)
+    bot.channels[channel].add_user(user, privs=priv)
 
 
 @sopel.module.event(events.RPL_WHOREPLY)
@@ -679,8 +704,9 @@ def _record_who(bot, channel, user, host, nick, account=None, away=None):
 @sopel.module.priority('high')
 @sopel.module.unblockable
 def recv_who(bot, trigger):
-    channel, user, host, _, nick, = trigger.args[1:6]
-    _record_who(bot, channel, user, host, nick)
+    channel, user, host, _, nick, status = trigger.args[1:7]
+    modes = ''.join([c for c in status if c in '~&@%+'])
+    _record_who(bot, channel, user, host, nick, modes=modes)
 
 
 @sopel.module.event(events.RPL_ENDOFWHO)
