@@ -126,8 +126,8 @@ class Bot(asynchat.async_chat):
         if text is not None:
             text = self.safe(text)
         try:
-            self.writing_lock.acquire()  # Blocking lock, can't send two things
-                                         # at a time
+            # Blocking lock, can't send two things at a time
+            self.writing_lock.acquire()
 
             # From RFC2812 Internet Relay Chat: Client Protocol
             # Section 2.3
@@ -155,6 +155,7 @@ class Bot(asynchat.async_chat):
             self.initiate_connect(host, port)
         except socket.error as e:
             stderr('Connection error: %s' % e)
+            self.handle_close()
 
     def initiate_connect(self, host, port):
         stderr('Connecting to %s:%s...' % (host, port))
@@ -200,6 +201,11 @@ class Bot(asynchat.async_chat):
         self.close()
 
     def handle_connect(self):
+        """
+        Connect to IRC server, handle TLS and authenticate
+        user if an account exists.
+        """
+        # handle potential TLS connection
         if self.config.core.use_ssl and has_ssl:
             if not self.config.core.verify_ssl:
                 self.ssl = ssl.wrap_socket(self.socket,
@@ -211,12 +217,30 @@ class Bot(asynchat.async_chat):
                                            suppress_ragged_eofs=True,
                                            cert_reqs=ssl.CERT_REQUIRED,
                                            ca_certs=self.ca_certs)
+                # connect to host specified in config first
                 try:
                     ssl.match_hostname(self.ssl.getpeercert(), self.config.core.host)
                 except ssl.CertificateError:
-                    stderr("Invalid certficate, hostname mismatch!")
-                    os.unlink(self.config.core.pid_file_path)
-                    os._exit(1)
+                    # the host in config and certificate don't match
+                    LOGGER.error("hostname mismatch between configuration and certificate")
+                    # check (via exception) if a CNAME matches as a fallback
+                    has_matched = False
+                    for hostname in self._get_cnames(self.config.core.host):
+                        try:
+                            ssl.match_hostname(self.ssl.getpeercert(), hostname)
+                            LOGGER.warning("using {0} instead of {1} for TLS connection"
+                                           .format(hostname, self.config.core.host))
+                            has_matched = True
+                            break
+                        except ssl.CertificateError:
+                            pass
+                    if not has_matched:
+                        # everything is broken
+                        stderr("Invalid certificate, hostname mismatch!")
+                        LOGGER.error("invalid certificate, no hostname matches")
+                        if hasattr(self.config.core, 'pid_file_path'):
+                            os.unlink(self.config.core.pid_file_path)
+                            os._exit(1)
             self.set_socket(self.ssl)
 
         # Request list of server capabilities. IRCv3 servers will respond with
@@ -224,12 +248,14 @@ class Bot(asynchat.async_chat):
         # 421 Unknown command, which we'll ignore
         self.write(('CAP', 'LS', '302'))
 
+        # authenticate account if needed
         if self.config.core.auth_method == 'server':
             password = self.config.core.auth_password
             self.write(('PASS', password))
         self.write(('NICK', self.nick))
         self.write(('USER', self.user, '+iw', self.nick), self.name)
 
+        # maintain connection
         stderr('Connected.')
         self.last_ping_time = datetime.now()
         timeout_check_thread = threading.Thread(target=self._timeout_check)
@@ -239,6 +265,26 @@ class Bot(asynchat.async_chat):
         ping_thread.daemon = True
         ping_thread.start()
 
+    def _get_cnames(self, domain):
+        """
+        Determine the CNAMEs for a given domain.
+
+        :param domain: domain to check
+        :type domain: str
+        :returns: list (of str)
+        """
+        import dns.resolver
+        cnames = []
+        try:
+            answer = dns.resolver.query(domain, "CNAME")
+        except dns.resolver.NoAnswer:
+            return []
+        for data in answer:
+            if isinstance(data, dns.rdtypes.ANY.CNAME.CNAME):
+                cname = data.to_text()[:-1]
+                cnames.append(cname)
+        return cnames
+
     def _timeout_check(self):
         while self.connected or self.connecting:
             if (datetime.now() - self.last_ping_time).seconds > int(self.config.core.timeout):
@@ -304,7 +350,7 @@ class Bot(asynchat.async_chat):
                 # Okay, let's try ISO8859-1
                 try:
                     data = unicode(data, encoding='iso8859-1')
-                except:
+                except UnicodeDecodeError:
                     # Discard line if encoding is unknown
                     return
 
@@ -325,7 +371,7 @@ class Bot(asynchat.async_chat):
         if pretrigger.event == 'PING':
             self.write(('PONG', pretrigger.args[-1]))
         elif pretrigger.event == 'ERROR':
-            LOGGER.error("ERROR recieved from server: %s", pretrigger.args[-1])
+            LOGGER.error("ERROR received from server: %s", pretrigger.args[-1])
             if self.hasquit:
                 self.close_when_done()
         elif pretrigger.event == '433':
