@@ -7,13 +7,15 @@ from __future__ import unicode_literals, absolute_import, print_function, divisi
 import re
 from sopel import web
 from sopel.module import commands, example
-import json
+import requests
+import xmltodict
 import sys
 
 if sys.version_info.major < 3:
-    from urllib import quote_plus
+    from urllib import quote_plus, unquote as _unquote
+    unquote = lambda s: _unquote(s.encode('utf-8')).decode('utf-8')
 else:
-    from urllib.parse import quote_plus
+    from urllib.parse import quote_plus, unquote
 
 
 def formatnumber(n):
@@ -23,28 +25,32 @@ def formatnumber(n):
         parts.insert(i, ',')
     return ''.join(parts)
 
-r_bing = re.compile(r'<h3><a href="([^"]+)"')
+
+r_bing = re.compile(r'<h2(?: class=" b_topTitle")?><a href="([^"]+)"')
 
 
-def bing_search(query, lang='en-GB'):
-    base = 'http://www.bing.com/search?mkt=%s&q=' % lang
-    bytes = web.get(base + query)
+def bing_search(query, lang='en-US'):
+    base = 'https://www.bing.com/search?mkt=%s&q=' % lang
+    bytes = requests.get(base + query).text
     m = r_bing.search(bytes)
     if m:
         return m.group(1)
 
-r_duck = re.compile(r'nofollow" class="[^"]+" href="(?!https?:\/\/r\.search\.yahoo)(.*?)">')
+
+r_duck = re.compile(r'nofollow" class="[^"]+" href="(?!(?:https?:\/\/r\.search\.yahoo)|(?:https?:\/\/duckduckgo\.com\/y\.js)(?:\/l\/\?kh=-1&amp;uddg=))(.*?)">')
 
 
 def duck_search(query):
     query = query.replace('!', '')
-    uri = 'http://duckduckgo.com/html/?q=%s&kl=uk-en' % query
-    bytes = web.get(uri,headers={'User-Agent':'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36'})
+    uri = 'https://duckduckgo.com/html/?q=%s&kl=us-en' % query
+    bytes = requests.get(uri, headers={'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36'}).text
     if 'web-result' in bytes:  # filter out the adds on top of the page
         bytes = bytes.split('web-result')[1]
     m = r_duck.search(bytes)
     if m:
-        return web.decode(m.group(1))
+        unquoted_m = unquote(m.group(1))
+        return web.decode(unquoted_m)
+
 
 # Alias google_search to duck_search
 google_search = duck_search
@@ -59,8 +65,11 @@ def duck_api(query):
     # if the query constains special charactares that aren't urlencoded.
     # So in order to always get a JSON response back the query is urlencoded
     query = quote_plus(query)
-    uri = 'http://api.duckduckgo.com/?q=%s&format=json&no_html=1&no_redirect=1' % query
-    results = json.loads(web.get(uri))
+    uri = 'https://api.duckduckgo.com/?q=%s&format=json&no_html=1&no_redirect=1' % query
+    try:
+        results = requests.get(uri).json()
+    except ValueError:
+        return None
     if results['Redirect']:
         return results['Redirect']
     else:
@@ -68,7 +77,10 @@ def duck_api(query):
 
 
 @commands('duck', 'ddg', 'g')
-@example('.duck privacy or .duck !mcwiki obsidian')
+# test for bad Unicode handling in py2
+@example('.duck grandorder.wiki chulainn alter', 'https://grandorder.wiki/Cú_Chulainn_(Alter)')
+# the last example is what .help displays
+@example('.duck sopel irc bot', r'https?:\/\/sopel\.chat\/?', re=True)
 def duck(bot, trigger):
     """Queries Duck Duck Go for the specified input."""
     query = trigger.group(2)
@@ -88,12 +100,31 @@ def duck(bot, trigger):
         bot.reply(uri)
         if 'last_seen_url' in bot.memory:
             bot.memory['last_seen_url'][trigger.sender] = uri
+    else:
+        msg = "No results found for '%s'." % query
+        if query.count('site:') >= 2:
+            # This check exists because of issue #1415. The git.io link will take the user there.
+            # (Better a sopel.chat link, but it's not set up to do that. This is shorter anyway.)
+            msg += " Try again with at most one 'site:' operator. See https://git.io/fpKtP for why."
+        bot.reply(msg)
+
+
+@commands('bing')
+@example('.bing sopel irc bot')
+def bing(bot, trigger):
+    """Queries Bing for the specified input."""
+    if not trigger.group(2):
+        return bot.reply('.bing what?')
+    query = trigger.group(2)
+    result = bing_search(query)
+    if result:
+        bot.say(result)
     else:
         bot.reply("No results found for '%s'." % query)
 
 
 @commands('search')
-@example('.search nerdfighter')
+@example('.search sopel irc bot')
 def search(bot, trigger):
     """Searches Bing and Duck Duck Go."""
     if not trigger.group(2):
@@ -115,13 +146,24 @@ def search(bot, trigger):
 
 
 @commands('suggest')
+@example('.suggest wikip', 'wikipedia')
+@example('.suggest ', 'No query term.')
+@example('.suggest lkashdfiauwgeaef', 'Sorry, no result.')
 def suggest(bot, trigger):
     """Suggest terms starting with given input"""
     if not trigger.group(2):
         return bot.reply("No query term.")
     query = trigger.group(2)
-    uri = 'http://websitedev.de/temp-bin/suggest.pl?q='
-    answer = web.get(uri + query.replace('+', '%2B'))
+    # Using Google isn't necessarily ideal, but at most they'll be able to build
+    # a composite profile of all users on a given instance, not a profile of any
+    # single user. This can be switched out as soon as someone finds (or builds)
+    # an alternative suggestion API.
+    uri = 'https://suggestqueries.google.com/complete/search?output=toolbar&hl=en&q='
+    answer = xmltodict.parse(requests.get(uri + query.replace('+', '%2B')).text)['toplevel']
+    try:
+        answer = answer['CompleteSuggestion'][0]['suggestion']['@data']
+    except TypeError:
+        answer = None
     if answer:
         bot.say(answer)
     else:
